//====----------------------- --- --   -      -
// MULTIUSER NETWORK MANAGEMENT
// TCP/IP and UDP support.
// 2000-2002, Vander Nunes
//======----------------------------- --- --   -      -

#ifndef _HTTP_H_

  #define _HTTP_H_
  #include <windows.h>
  #include <stdio.h>

  // size of socket buffer
  #define BUFFER_SIZE 1024

  class CNetClient;

  // download stages enumeration
  enum enHttpDownStage { dsIdle, dsConn, dsReq, dsRecv, dsFail };

  // http get file request
//  #define HTTP_GETREQUEST "GET %s%s%s HTTP/1.1\nUser-Agent: VirtwareA1\nHost: %s\nConnection: close\r\n\r\n"
  #define HTTP_GETREQUEST "GET %s%s%s HTTP/1.1\nHost: %s\nConnection: close\n\n"

  // request OK, file found
  #define HTTP_GETOK      "200 OK"

  // request OK, file found but in another place
  // (must parse response and build the request again)
  // ### not used yet
  #define HTTP_GETOKREDIR "302 Found"

  // end of the http header
  #define HTTP_HEADEREND "\r\n\r\n"

  // user's callback
  #define USR_HTTPCALLBACK(userProc) void (*userProc)(CHttp *pHttp, void *pUserData, enHttpDownStage Stage, DWORD dwBytesDownloaded)


  // ----------------------------------------------------------------------------------------------
  //
  // CHttp download class
  //
  // ----------------------------------------------------------------------------------------------
  class CHttp
  {
    friend void HttpClientCallback(CNetClient *pClient, void *pData, int iMsg);
    friend LRESULT CALLBACK HttpWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);

    private:

      // download stage
      enHttpDownStage           m_Stage;

      // hidden window to handle timer timeout
      HWND                      m_hWnd;

      // flag if a download is in progress
      BOOL                      m_bWorking;

      // socket
      CNetClient               *m_pClient;

      // host, file and target
      char                      m_szHost[MAX_PATH];
      char                      m_szFile[MAX_PATH];
      char                      m_szTarget[MAX_PATH];

      // proxy info
      BOOL                      m_bUseProxy;
      char                      m_szProxyAddr[33];
      int                       m_iProxyPort;

      // file
      FILE                     *m_hFile;
      DWORD                     m_dwBytesDownloaded;

      // user callback
      USR_HTTPCALLBACK(m_userProc);
      void                     *m_pUserData;

      // internal callback
      void                      Callback(CNetClient *pClient, int iMsg);

      // internal search for string
      int CHttp::strpos(char *szStr, char *szString);

      void                      CloseDownload();
      void                      Fail();

    public:

      CHttp(USR_HTTPCALLBACK(userProc), void *pUserData);
      virtual ~CHttp();

      // ------------------------------------------------------------------------------------------
      // download a file from a url
      // ------------------------------------------------------------------------------------------

      BOOL                      Download(char *szHost, char *szFile, char *szTarget, int iPort=80, char *szProxyAddr=NULL, int iProxyPort=0);

      // ------------------------------------------------------------------------------------------
      // stop a download
      // ------------------------------------------------------------------------------------------

      void                      Stop();

      // ------------------------------------------------------------------------------------------
      // return the actual download stage
      // ------------------------------------------------------------------------------------------

      enHttpDownStage           Stage() { return m_Stage; }

      // ------------------------------------------------------------------------------------------
      // tell if a download is in progress
      // ------------------------------------------------------------------------------------------

      BOOL                      Working() { return m_bWorking; }
  };

#endif
